#!/usr/bin/env python
# -*- coding: utf-8 -*-

# intended to be piped to `odoo shell`
# example: cat generate_dependencies.py | ./odoo-bin shell -d 12.0 --db-filter=12.0 --log-level=error > private/openerp_enterprise/data/modules_dependencies/12.0-modules.xml

from collections import OrderedDict
from lxml import etree
from odoo.tools import ustr

try:
    from odoo.addons.base.models.ir_qweb import render as nodb_render
    QWeb = object
except ImportError:
    nodb_render = None
    try:
        from odoo.addons.base.ir.ir_qweb.qweb import QWeb
    except ImportError:
        from odoo.addons.base.models.qweb import QWeb


extra_apps = (
    'delivery_dhl', 'delivery_ups', 'delivery_usps',
    'delivery_fedex', 'delivery_temando', 'sale_ebay', 'crm_voip',
    'account_voucher', 'delivery_bpost', 'delivery_easypost', 'sale_amazon'
)

# /trial starting module - copy all apps of subscription-widget-container from oe/o_e_template.xml
# application=ture is not enough, since we can starts with website_mass_mailing, we need to know
# their dependencies and know if it is free combination
extra_apps += (
    'website', 'website_sale', 'website_blog', 'website_forum', 'website_slides', 'website_livechat', 'crm',
    'sale_management', 'account', 'point_of_sale', 'sign', 'sale_subscription', 'sale_renting', 'industry_fsm',
    'account_accountant', 'account_consolidation', 'project', 'timesheet_grid', 'helpdesk', 'stock', 'purchase',
    'documents', 'mrp', 'mrp_plm', 'maintenance', 'quality_control', 'repair', 'mass_mailing', 'mass_mailing_sms',
    'marketing_automation', 'website_event', 'survey', 'social', 'appointment', 'website_mass_mailing', 'hr',
    'hr_recruitment', 'hr_referral', 'hr_holidays', 'hr_expense', 'planning', 'hr_appraisal', 'fleet', 'approvals',
    'lunch', 'web_studio'
)

env = env  # noqa

env.cr.execute("""
    select (regexp_matches(latest_version, '^\d+\.0|^saas~\d+\.\d+|saas~\d+'))[1]
      from ir_module_module
     where name='base'
""")

version = env.cr.fetchone()[0].replace('~', '-')
xversion = version.replace('.', '_')

query = """
      WITH RECURSIVE deps (mod_id, dep_name) AS (
              SELECT module_id, name from ir_module_module_dependency
               UNION
              SELECT deps.mod_id, d.name from ir_module_module m
                JOIN deps ON deps.dep_name = m.name
                JOIN ir_module_module_dependency d on (d.module_id = m.id)
        )
        SELECT m.name, m.shortdesc::json->'en_US',
               array_remove(array_agg(CASE WHEN o.application=true THEN d.dep_name END ORDER BY d.dep_name), NULL)
          FROM ir_module_module m
     LEFT JOIN deps d ON (d.mod_id = m.id)
     LEFT JOIN ir_module_module o ON (d.dep_name = o.name)
         WHERE m.application = true or m.name in %s
      GROUP BY m.name, m.shortdesc
      ORDER BY m.name
"""
env.cr.execute(query, (extra_apps,))
apps = OrderedDict()
for name, desc, deps in env.cr.fetchall():
    apps[name] = [desc, deps]

tmpl = """\
<odoo>
  <![CDATA[<!--This is an autogenerated file; changes may be lost -->]]>
  <data noupdate="1">
    <record model="openerp.enterprise.module.version" t-attf-id="mod_ver_{{xversion}}">
      <field name="name"><t t-esc="version"/></field>
      <field name="sequence"><t t-esc="int(''.join([c for c in version if c.isdigit()]))"/></field>
    </record>
    <t t-foreach="apps" t-as="app">
    <record model="openerp.enterprise.database.module" t-attf-id="mod_{{app}}_{{xversion}}">
      <field name="name"><t t-esc="apps[app][0]"/></field>
      <field name="technical_name"><t t-esc="app"/></field>
      <field name="version_id" t-attf-ref="mod_ver_{{xversion}}"/>
    </record>
    </t>
    <t t-foreach="apps" t-as="app">
    <record model="openerp.enterprise.database.module" t-attf-id="mod_{{app}}_{{xversion}}">
      <field name="dependency_ids"
             t-att-eval="'[(6, 0, [' + ', '.join('ref(\\'mod_%s_%s\\')' % (d, xversion) for d in apps[app][1]) + '])]'"/>
    </record>
    </t>
  </data>
</odoo>
"""

qweb = QWeb()

if nodb_render: # saas-15.1+
    xml = nodb_render(f'{xversion}-dependencies', {
        'apps': apps,
        'xversion': xversion,
        'version': version,
    }, lambda *arg, **kw: (etree.fromstring(tmpl), ''))
elif hasattr(qweb, 'render'):
    xml = QWeb().render(etree.fromstring(tmpl), locals())
else:
    xml = QWeb()._render(etree.fromstring(tmpl), {
        'apps': apps,
        'xversion': xversion,
        'version': version,
    }) # 14.0+

print(ustr(xml).replace("&#39;", "'"))
